import numpy as np
from PySide6 import QtCore, QtGui, QtWidgets

from ..ui import resource_rc
from ..ui.vrtt_widget import Ui_VRTT_Widget


class VRTTWidget(Ui_VRTT_Widget, QtWidgets.QWidget):
    def __init__(
        self,
        lower=0,
        upper=0,
        text="",
        conversion=None,
        mode="text",
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self.text.setText(text)

        self.lower.setMaximum(np.inf)
        self.lower.setMinimum(-np.inf)
        self.lower.setValue(lower)

        self.upper.setMaximum(np.inf)
        self.upper.setMinimum(-np.inf)
        self.upper.setValue(upper)

        self.conversion = conversion

        self.mode_switch.currentIndexChanged.connect(self.mode.setCurrentIndex)

        if mode == "text":
            self.mode_switch.setCurrentIndex(0)
        else:
            self.mode_switch.setCurrentIndex(1)

        self.conversion_btn.clicked.connect(self.edit_conversion)

    def edit_conversion(self):
        dlg = ConversionEditor(
            f"Raw=[{self.lower.value()}, {self.upper.value()}) referenced",
            self.conversion,
            parent=self,
        )
        dlg.exec_()
        if dlg.pressed_button == "apply":
            self.conversion = dlg.conversion()

    def reference(self):
        if self.mode.currentIndex() == 0:
            return self.text.text().strip().encode("utf-8")
        else:
            return self.conversion


from ..dialogs.conversion_editor import ConversionEditor



#!/usr/bin/env python
from pathlib import Path
import shutil
import tempfile
import unittest

import numpy as np

from asammdf import MDF, Signal
import asammdf.blocks.v2_v3_constants as v23c
import asammdf.blocks.v4_constants as v4c


class TestEndianess(unittest.TestCase):
    tempdir = None

    @classmethod
    def setUpClass(cls):
        cls.tempdir = tempfile.TemporaryDirectory()

    def test_mixed(self):
        t = np.arange(15, dtype="<f8")

        s1 = Signal(np.frombuffer(b"\x00\x00\x00\x02" * 15, dtype=">u4"), t, name="Motorola")

        s2 = Signal(np.frombuffer(b"\x04\x00\x00\x00" * 15, dtype="<u4"), t, name="Intel")

        for version in ("3.30", "4.10"):
            mdf = MDF(version=version)
            mdf.append([s1, s2], common_timebase=True)
            outfile = mdf.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)
            mdf.close()

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("Motorola").samples, [2] * 15)
                assert np.array_equal(mdf.get("Intel").samples, [4] * 15)

        for version in ("3.30", "4.10"):
            mdf = MDF(version=version)
            mdf.append([s2, s1], common_timebase=True)
            outfile = mdf.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)
            mdf.close()

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("Motorola").samples, [2] * 15)
                assert np.array_equal(mdf.get("Intel").samples, [4] * 15)

    def test_not_aligned_mdf_v3(self):
        t = np.arange(15, dtype="<f8")

        s1 = Signal(np.frombuffer(b"\x00\x00\x3F\x02" * 15, dtype=">u4"), t, name="Motorola")

        s2 = Signal(np.frombuffer(b"\x04\xF8\x00\x00" * 15, dtype="<u4"), t, name="Intel")

        s3 = Signal(
            np.frombuffer(b"\xBB\x55" * 2 * 15, dtype="<u4"),
            t,
            name="NotAlignedMotorola",
        )

        s4 = Signal(np.frombuffer(b"\xBB\x55" * 2 * 15, dtype="<u4"), t, name="NotAlignedIntel")

        with MDF(version="3.30") as mdf_source:
            mdf_source.append([s1, s2, s3, s4], common_timebase=True)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.start_offset = 24 + t.itemsize * 8
            ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 16

            ch4 = mdf_source.groups[0].channels[4]
            ch4.start_offset = 24 + t.itemsize * 8
            ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 16

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x0204] * 15)
                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.start_offset = 16 + t.itemsize * 8
            ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 24

            ch4 = mdf_source.groups[0].channels[4]
            ch4.start_offset = 16 + t.itemsize * 8
            ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 24

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15)

                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x04023F] * 15)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.start_offset = 16 + t.itemsize * 8 + 5
            ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 21

            ch4 = mdf_source.groups[0].channels[4]
            ch4.start_offset = 16 + t.itemsize * 8 + 6
            ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 21

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(
                    mdf.get("NotAlignedMotorola").samples,
                    [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                )

                assert np.array_equal(
                    mdf.get("NotAlignedIntel").samples,
                    [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                )

    def test_not_aligned_mdf_v4(self):
        t = np.arange(15, dtype="<f8")

        s1 = Signal(np.frombuffer(b"\x00\x00\x3F\x02" * 15, dtype=">u4"), t, name="Motorola")

        s2 = Signal(np.frombuffer(b"\x04\xF8\x00\x00" * 15, dtype="<u4"), t, name="Intel")

        s3 = Signal(
            np.frombuffer(b"\xBB\x55" * 2 * 15, dtype="<u4"),
            t,
            name="NotAlignedMotorola",
        )

        s4 = Signal(np.frombuffer(b"\xBB\x55" * 2 * 15, dtype="<u4"), t, name="NotAlignedIntel")

        with MDF(version="4.11") as mdf_source:
            mdf_source.append([s1, s2, s3, s4], common_timebase=True)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.byte_offset = 3 + t.itemsize
            ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 16

            ch4 = mdf_source.groups[0].channels[4]
            ch4.byte_offset = 3 + t.itemsize
            ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 16

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x0204] * 15)

                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.byte_offset = 2 + t.itemsize
            ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 24

            ch4 = mdf_source.groups[0].channels[4]
            ch4.byte_offset = 2 + t.itemsize
            ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 24

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15)

                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x04023F] * 15)

            ch3 = mdf_source.groups[0].channels[3]
            ch3.byte_offset = 2 + t.itemsize
            ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
            ch3.bit_count = 21
            ch3.bit_offset = 5

            ch4 = mdf_source.groups[0].channels[4]
            ch4.byte_offset = 2 + t.itemsize
            ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
            ch4.bit_count = 21
            ch4.bit_offset = 6

            outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

            with MDF(outfile) as mdf:
                assert np.array_equal(
                    mdf.get("NotAlignedMotorola").samples,
                    [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                )

                assert np.array_equal(
                    mdf.get("NotAlignedIntel").samples,
                    [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                )

    def test_overlapping_channels_mdf_v3(self):
        t = np.arange(15, dtype="<f8")

        s1 = Signal(
            np.frombuffer(b"\x00\x00\x3F\x02\x04\xF8\x00\x00" * 15, dtype="<u8"),
            t,
            name="Parent",
        )

        s2 = Signal(
            np.frombuffer(b"\x04\xF8\x00\x00" * 15, dtype="<u4"),
            t,
            name="OverlappingMotorola",
        )

        s3 = Signal(
            np.frombuffer(b"\x04\x00\x00\x00" * 15, dtype="<u4"),
            t,
            name="OverlappingIntel",
        )

        with MDF(version="3.30") as mdf_source:
            mdf_source.append([s1, s2, s3], common_timebase=True)

            for parent_data_type in (
                v23c.DATA_TYPE_DOUBLE_MOTOROLA,
                v23c.DATA_TYPE_SIGNED_MOTOROLA,
                v23c.DATA_TYPE_UNSIGNED_MOTOROLA,
                v23c.DATA_TYPE_DOUBLE_INTEL,
                v23c.DATA_TYPE_SIGNED_INTEL,
                v23c.DATA_TYPE_UNSIGNED_INTEL,
            ):
                parent = mdf_source.groups[0].channels[1]
                parent.data_type = parent_data_type

                ch3 = mdf_source.groups[0].channels[2]
                ch3.start_offset = 24 + t.itemsize * 8
                ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 16

                ch4 = mdf_source.groups[0].channels[3]
                ch4.start_offset = 24 + t.itemsize * 8
                ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 16

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(mdf.get("OverlappingMotorola").samples, [0x0204] * 15)

                    assert np.array_equal(mdf.get("OverlappingIntel").samples, [0x0402] * 15)

                ch3 = mdf_source.groups[0].channels[2]
                ch3.start_offset = 16 + t.itemsize * 8
                ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 24

                ch4 = mdf_source.groups[0].channels[3]
                ch4.start_offset = 16 + t.itemsize * 8
                ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 24

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15)

                    assert np.array_equal(mdf.get("OverlappingIntel").samples, [0x04023F] * 15)

                ch3 = mdf_source.groups[0].channels[2]
                ch3.start_offset = 16 + t.itemsize * 8 + 5
                ch3.data_type = v23c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 21

                ch4 = mdf_source.groups[0].channels[3]
                ch4.start_offset = 16 + t.itemsize * 8 + 6
                ch4.data_type = v23c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 21

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(
                        mdf.get("OverlappingMotorola").samples,
                        [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                    )

                    assert np.array_equal(
                        mdf.get("OverlappingIntel").samples,
                        [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                    )

    def test_overlapping_channels_mdf_v4(self):
        t = np.arange(15, dtype="<f8")

        s1 = Signal(
            np.frombuffer(b"\x00\x00\x3F\x02\x04\xF8\x00\x00" * 15, dtype="<u8"),
            t,
            name="Parent",
        )

        s2 = Signal(
            np.frombuffer(b"\x04\xF8\x00\x00" * 15, dtype="<u4"),
            t,
            name="OverlappingMotorola",
        )

        s3 = Signal(
            np.frombuffer(b"\x04\x00\x00\x00" * 15, dtype="<u4"),
            t,
            name="OverlappingIntel",
        )

        with MDF(version="4.11") as mdf_source:
            mdf_source.append([s1, s2, s3], common_timebase=True)

            for parent_data_type in (
                v4c.DATA_TYPE_REAL_MOTOROLA,
                v4c.DATA_TYPE_SIGNED_MOTOROLA,
                v4c.DATA_TYPE_UNSIGNED_MOTOROLA,
                v4c.DATA_TYPE_REAL_INTEL,
                v4c.DATA_TYPE_SIGNED_INTEL,
                v4c.DATA_TYPE_UNSIGNED_INTEL,
            ):
                parent = mdf_source.groups[0].channels[1]
                parent.data_type = parent_data_type

                ch3 = mdf_source.groups[0].channels[2]
                ch3.byte_offset = 3 + t.itemsize
                ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 16
                ch3.bit_offset = 0

                ch4 = mdf_source.groups[0].channels[3]
                ch4.byte_offset = 3 + t.itemsize
                ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 16
                ch4.bit_offset = 0

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(mdf.get("OverlappingMotorola").samples, [0x0204] * 15)

                    assert np.array_equal(mdf.get("OverlappingIntel").samples, [0x0402] * 15)

                ch3 = mdf_source.groups[0].channels[2]
                ch3.byte_offset = 2 + t.itemsize
                ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 24
                ch3.bit_offset = 0

                ch4 = mdf_source.groups[0].channels[3]
                ch4.byte_offset = 2 + t.itemsize
                ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 24
                ch4.bit_offset = 0

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15)

                    assert np.array_equal(mdf.get("OverlappingIntel").samples, [0x04023F] * 15)

                ch3 = mdf_source.groups[0].channels[2]
                ch3.byte_offset = 2 + t.itemsize
                ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 21
                ch3.bit_offset = 5

                ch4 = mdf_source.groups[0].channels[3]
                ch4.byte_offset = 2 + t.itemsize
                ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 21
                ch4.bit_offset = 6

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(
                        mdf.get("OverlappingMotorola").samples,
                        [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                    )

                    assert np.array_equal(
                        mdf.get("OverlappingIntel").samples,
                        [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                    )

                ch3 = mdf_source.groups[0].channels[2]
                ch3.byte_offset = 3 + t.itemsize
                ch3.data_type = v4c.DATA_TYPE_UNSIGNED_MOTOROLA
                ch3.bit_count = 2
                ch3.bit_offset = 1

                ch4 = mdf_source.groups[0].channels[3]
                ch4.byte_offset = 3 + t.itemsize
                ch4.data_type = v4c.DATA_TYPE_UNSIGNED_INTEL
                ch4.bit_count = 2
                ch4.bit_offset = 1

                outfile = mdf_source.save(Path(TestEndianess.tempdir.name) / f"out", overwrite=True)

                with MDF(outfile) as mdf:
                    assert np.array_equal(mdf.get("OverlappingMotorola").samples, [0x1] * 15)

                    assert np.array_equal(mdf.get("OverlappingIntel").samples, [0x1] * 15)


if __name__ == "__main__":
    unittest.main()



from functools import partial

from natsort import natsorted
import numpy as np
from pyqtgraph import functions as fn
from PySide6 import QtCore, QtGui, QtWidgets

from ..ui import resource_rc
from ..ui.signal_scale import Ui_ScaleDialog
from ..utils import BLUE
from .plot import PlotSignal

PLOT_HEIGTH = 600  # pixels
TEXT_WIDTH = 50  # pixels


class ScaleDialog(Ui_ScaleDialog, QtWidgets.QDialog):
    def __init__(self, signals, y_range, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)
        self.signals = signals
        self._inhibit = True

        self.signal.addItems(list(natsorted(signals)))
        self.signal.currentTextChanged.connect(self.signal_selected)

        self.setWindowTitle("Y scale configuration")

        self.scaling.setMinimum(0.000001)
        self.scaling.setMaximum(np.inf)
        self.target_min.setMinimum(-np.inf)
        self.target_min.setMaximum(np.inf)
        self.target_max.setMinimum(-np.inf)
        self.target_max.setMaximum(np.inf)
        self.offset.setMinimum(-np.inf)
        self.offset.setMaximum(np.inf)

        self.signal_selected(self.signal.currentText())

        self.apply_btn.clicked.connect(self.apply)
        self.cancel_btn.clicked.connect(self.cancel)
        self.zoom_in_btn.clicked.connect(self.zoom_in)
        self.zoom_out_btn.clicked.connect(self.zoom_out)
        self.fit_btn.clicked.connect(self.fit)
        self.shift_up_btn.clicked.connect(partial(self.shift, step=1))
        self.shift_down_btn.clicked.connect(partial(self.shift, step=-1))
        self.fast_shift_up_btn.clicked.connect(partial(self.shift, step=10))
        self.fast_shift_down_btn.clicked.connect(partial(self.shift, step=-10))

        self.apply_btn.setAutoDefault(False)
        self.cancel_btn.setAutoDefault(False)
        self.zoom_in_btn.setAutoDefault(False)
        self.zoom_out_btn.setAutoDefault(False)
        self.fit_btn.setAutoDefault(False)
        self.shift_up_btn.setAutoDefault(False)
        self.shift_down_btn.setAutoDefault(False)
        self.fast_shift_up_btn.setAutoDefault(False)
        self.fast_shift_down_btn.setAutoDefault(False)

        self.offset.valueChanged.connect(self.draw_plot)
        self.scaling.valueChanged.connect(self.draw_plot)
        self.target_max.valueChanged.connect(self.set_target)
        self.target_min.valueChanged.connect(self.set_target)

        canvas = QtGui.QPixmap(PLOT_HEIGTH + 3 * TEXT_WIDTH, PLOT_HEIGTH)
        canvas.fill(QtCore.Qt.GlobalColor.black)
        self.plot.setPixmap(canvas)
        self.plot.setFocus()

        self._inhibit = False

        self.signal.setCurrentIndex(0)
        bottom, top = y_range

        if top == bottom:
            top += 1
            bottom -= 1
            scaling = 2
        else:
            scaling = top - bottom

        self.scaling.setValue(scaling)
        self.offset.setValue(-bottom * 100 / scaling)

    def set_target(self, *args):
        original_change = not self._inhibit

        if original_change:
            self._inhibit = True

            if self.target_max.value() < self.target_min.value():
                val = self.target_min.value()

                self.target_min.setValue(self.target_max.value())
                self.target_max.setValue(val)

            self._inhibit = False

        self.draw_plot()

    def apply(self, *args):
        self.accept()

    def cancel(self):
        self.reject()

    def zoom_in(self, *args):
        self.scaling.setValue(self.scaling.value() * 0.9)

    def zoom_out(self, *args):
        self.scaling.setValue(self.scaling.value() * 1.1)

    def fit(self, *args):
        scaling = (self.target_max.value() - self.target_min.value()) or 2
        pos = -(100 * self.target_min.value() / scaling)
        self.offset.setValue(pos)
        self.scaling.setValue(scaling)

    def draw_plot(self, *args):
        offset = self.offset.value()
        scale = self.scaling.value()

        y_bottom = -offset * scale / 100
        y_top = y_bottom + scale

        self.y_bottom.setText(f"{y_bottom:.3f}")
        self.y_top.setText(f"{y_top:.3f}")

        canvas = self.plot.pixmap()
        canvas.fill(QtCore.Qt.GlobalColor.black)

        if self.samples is None:
            self.target_max.setEnabled(True)
            self.target_min.setEnabled(True)
        else:
            self.target_max.setEnabled(False)
            self.target_min.setEnabled(False)

        if self.samples is None:
            x = np.linspace(0, 2 * np.pi * 10, PLOT_HEIGTH)
            amp = (self.target_max.value() - self.target_min.value()) / 2
            off = (self.target_max.value() + self.target_min.value()) / 2
            y = np.sin(x) * amp + off
            x = np.arange(50, 50 + PLOT_HEIGTH)
        else:
            x = self.samples.plot_timestamps
            y = self.samples.plot_samples.copy()

        ys = y_top
        y_scale = (y_top - y_bottom) / PLOT_HEIGTH

        ys = ys + y_scale

        y = (ys - y) / y_scale

        polygon = fn.create_qpolygonf(len(x))
        ndarray = fn.ndarray_from_qpolygonf(polygon)

        ndarray[:, 0] = x
        ndarray[:, 1] = y

        painter = QtGui.QPainter(canvas)
        pen = QtGui.QPen(QtCore.Qt.GlobalColor.white)
        painter.setPen(pen)

        step = PLOT_HEIGTH // 10
        for i, x in enumerate(range(0, PLOT_HEIGTH + step, step)):
            if i == 0:
                painter.drawText(5, x + 15, f"{100 - 10 * i}%")
            elif i == 10:
                painter.drawText(5, x - 5, f"{100 - 10 * i}%")
            else:
                painter.drawText(5, x + 6, f"{100-10*i}%")

        painter.drawText(PLOT_HEIGTH + TEXT_WIDTH, 15, f"{y_top:.3f}")
        painter.drawText(
            PLOT_HEIGTH + TEXT_WIDTH,
            PLOT_HEIGTH - 5,
            f"{y_bottom:.3f}",
        )

        painter.setClipping(True)
        painter.setClipRect(QtCore.QRect(TEXT_WIDTH, 0, PLOT_HEIGTH, PLOT_HEIGTH))

        pen.setStyle(QtCore.Qt.PenStyle.DotLine)
        painter.setPen(pen)
        for i, x in enumerate(range(0, PLOT_HEIGTH + step, step)):
            painter.drawLine(0, x, PLOT_HEIGTH + 2 * TEXT_WIDTH, x)

        pen = QtGui.QPen(BLUE)
        pen.setWidth(2)
        painter.setPen(pen)
        painter.drawPolyline(polygon)

        pen = QtGui.QPen("#18e223")
        pen.setWidth(2)
        pen.setStyle(QtCore.Qt.PenStyle.DashDotDotLine)
        painter.setPen(pen)
        offset = int(PLOT_HEIGTH - self.offset.value() * PLOT_HEIGTH / 100)
        painter.drawLine(0, offset, PLOT_HEIGTH + 2 * TEXT_WIDTH, offset)

        painter.end()

        self.plot.setPixmap(canvas)

    def shift(self, *args, step=1):
        self.offset.stepBy(step)

    def keyPressEvent(self, event):
        key = event.key()
        modifiers = event.modifiers()

        if key == QtCore.Qt.Key.Key_I and modifiers == QtCore.Qt.KeyboardModifier.NoModifier:
            self.zoom_in()
        elif key == QtCore.Qt.Key.Key_O and modifiers == QtCore.Qt.KeyboardModifier.NoModifier:
            self.zoom_out()
        elif key == QtCore.Qt.Key.Key_F and modifiers == QtCore.Qt.KeyboardModifier.NoModifier:
            self.fit()
        else:
            super().keyPressEvent(event)

    def signal_selected(self, name):
        signal = self.signals[name].copy()
        signal.flags &= ~signal.Flags.computed
        signal.computation = {}

        signal = PlotSignal(signal)
        if not isinstance(signal.min, str):
            self.target_max.setValue(signal.max)
            self.target_min.setValue(signal.min)

            if len(signal):
                self.samples = signal
                self.samples.trim(
                    signal.timestamps[0],
                    signal.timestamps[-1],
                    PLOT_HEIGTH,
                )
                self.samples.plot_timestamps -= self.samples.plot_timestamps[0]
                x_scale = self.samples.plot_timestamps[-1] / PLOT_HEIGTH
                self.samples.plot_timestamps /= x_scale
                self.samples.plot_timestamps += TEXT_WIDTH

            else:
                self.samples = None
        else:
            self.samples = None

        if self.samples is None:
            self.target_max.setEnabled(True)
            self.target_min.setEnabled(True)
        else:
            self.target_max.setEnabled(False)
            self.target_min.setEnabled(False)

        self.draw_plot()



from PySide6 import QtCore, QtWidgets

from ..ui import resource_rc
from ..ui.search_widget import Ui_SearchWidget


class SearchWidget(Ui_SearchWidget, QtWidgets.QWidget):
    selectionChanged = QtCore.Signal()

    def __init__(self, sorted_keys, channels_db, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)
        self.channels_db = channels_db

        self.matches = 0
        self.current_index = 1
        self.entries = []

        completer = QtWidgets.QCompleter(sorted_keys, self)
        completer.setCaseSensitivity(QtCore.Qt.CaseSensitivity.CaseInsensitive)
        completer.setModelSorting(QtWidgets.QCompleter.ModelSorting.CaseInsensitivelySortedModel)
        completer.setFilterMode(QtCore.Qt.MatchFlag.MatchContains)
        self.search.setCompleter(completer)

        self.search.textChanged.connect(self.display_results)

        self.up_btn.clicked.connect(self.up)
        self.down_btn.clicked.connect(self.down)

    def down(self, event):
        if self.matches:
            self.current_index += 1
            if self.current_index >= self.matches:
                self.current_index = 0
            self.label.setText(f"{self.current_index + 1} of {self.matches}")
            self.selectionChanged.emit()

    def up(self, event):
        if self.matches:
            self.current_index -= 1
            if self.current_index < 0:
                self.current_index = self.matches - 1
            self.label.setText(f"{self.current_index + 1} of {self.matches}")
            self.selectionChanged.emit()

    def set_search_option(self, option):
        if option == "Match start":
            self.search.completer().setFilterMode(QtCore.Qt.MatchFlag.MatchStartsWith)
        elif option == "Match contains":
            self.search.completer().setFilterMode(QtCore.Qt.MatchFlag.MatchContains)

    def display_results(self, text):
        channel_name = text.strip()
        if channel_name in self.channels_db:
            self.entries = self.channels_db[channel_name]
            self.matches = len(self.entries)
            self.label.setText(f"1 of {self.matches}")
            self.current_index = 0
            self.selectionChanged.emit()

        else:
            self.label.setText("No match")
            self.matches = 0
            self.current_index = 0
            self.entries = []



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'define_channel_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDialog, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QLabel,
    QLineEdit, QPlainTextEdit, QPushButton, QRadioButton,
    QScrollArea, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)
from . import resource_rc

class Ui_ComputedChannel(object):
    def setupUi(self, ComputedChannel):
        if not ComputedChannel.objectName():
            ComputedChannel.setObjectName(u"ComputedChannel")
        ComputedChannel.resize(723, 552)
        ComputedChannel.setMaximumSize(QSize(16777215, 16777215))
        icon = QIcon()
        icon.addFile(u":/plus.png", QSize(), QIcon.Normal, QIcon.Off)
        ComputedChannel.setWindowIcon(icon)
        ComputedChannel.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(ComputedChannel)
        self.gridLayout.setSpacing(2)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(4, 4, 4, 4)
        self.comment = QPlainTextEdit(ComputedChannel)
        self.comment.setObjectName(u"comment")

        self.gridLayout.addWidget(self.comment, 5, 1, 1, 4)

        self.apply_btn = QPushButton(ComputedChannel)
        self.apply_btn.setObjectName(u"apply_btn")

        self.gridLayout.addWidget(self.apply_btn, 7, 3, 1, 1)

        self.groupBox_3 = QGroupBox(ComputedChannel)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.gridLayout_8 = QGridLayout(self.groupBox_3)
        self.gridLayout_8.setSpacing(2)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_8.setContentsMargins(4, 4, 4, 4)
        self.horizontalSpacer_8 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_8.addItem(self.horizontalSpacer_8, 1, 2, 1, 1)

        self.trigger_interval = QDoubleSpinBox(self.groupBox_3)
        self.trigger_interval.setObjectName(u"trigger_interval")
        self.trigger_interval.setDecimals(6)
        self.trigger_interval.setMinimum(0.000001000000000)
        self.trigger_interval.setMaximum(10.000000000000000)
        self.trigger_interval.setSingleStep(0.100000000000000)
        self.trigger_interval.setValue(0.010000000000000)

        self.gridLayout_8.addWidget(self.trigger_interval, 1, 1, 1, 1)

        self.trigger_channel = QLineEdit(self.groupBox_3)
        self.trigger_channel.setObjectName(u"trigger_channel")

        self.gridLayout_8.addWidget(self.trigger_channel, 4, 1, 1, 2)

        self.triggering_on_channel = QRadioButton(self.groupBox_3)
        self.triggering_on_channel.setObjectName(u"triggering_on_channel")

        self.gridLayout_8.addWidget(self.triggering_on_channel, 4, 0, 1, 1)

        self.triggering_on_interval = QRadioButton(self.groupBox_3)
        self.triggering_on_interval.setObjectName(u"triggering_on_interval")

        self.gridLayout_8.addWidget(self.triggering_on_interval, 1, 0, 1, 1)

        self.trigger_search_btn = QPushButton(self.groupBox_3)
        self.trigger_search_btn.setObjectName(u"trigger_search_btn")
        icon1 = QIcon()
        icon1.addFile(u":/search.png", QSize(), QIcon.Normal, QIcon.Off)
        self.trigger_search_btn.setIcon(icon1)

        self.gridLayout_8.addWidget(self.trigger_search_btn, 4, 3, 1, 1)

        self.triggering_on_all = QRadioButton(self.groupBox_3)
        self.triggering_on_all.setObjectName(u"triggering_on_all")
        self.triggering_on_all.setChecked(True)

        self.gridLayout_8.addWidget(self.triggering_on_all, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox_3, 0, 0, 1, 5)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_3, 7, 1, 1, 1)

        self.cancel_btn = QPushButton(ComputedChannel)
        self.cancel_btn.setObjectName(u"cancel_btn")

        self.gridLayout.addWidget(self.cancel_btn, 7, 4, 1, 1)

        self.label_6 = QLabel(ComputedChannel)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout.addWidget(self.label_6, 6, 0, 1, 1)

        self.label_8 = QLabel(ComputedChannel)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.gridLayout.addWidget(self.label_8, 5, 0, 1, 1)

        self.unit = QLineEdit(ComputedChannel)
        self.unit.setObjectName(u"unit")

        self.gridLayout.addWidget(self.unit, 6, 1, 1, 1)

        self.label_7 = QLabel(ComputedChannel)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout.addWidget(self.label_7, 3, 0, 1, 1)

        self.name = QLineEdit(ComputedChannel)
        self.name.setObjectName(u"name")

        self.gridLayout.addWidget(self.name, 3, 1, 1, 4)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_7, 7, 2, 1, 1)

        self.groupBox = QGroupBox(ComputedChannel)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout = QVBoxLayout(self.groupBox)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(self.groupBox)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setFrameShape(QFrame.NoFrame)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setWidgetResizable(True)
        self.args_widget = QWidget()
        self.args_widget.setObjectName(u"args_widget")
        self.args_widget.setGeometry(QRect(0, 0, 713, 69))
        self.arg_layout = QGridLayout(self.args_widget)
        self.arg_layout.setSpacing(2)
        self.arg_layout.setObjectName(u"arg_layout")
        self.arg_layout.setContentsMargins(9, 4, 4, 4)
        self.show_definition_btn = QPushButton(self.args_widget)
        self.show_definition_btn.setObjectName(u"show_definition_btn")
        icon2 = QIcon()
        icon2.addFile(u":/info.png", QSize(), QIcon.Normal, QIcon.Off)
        self.show_definition_btn.setIcon(icon2)

        self.arg_layout.addWidget(self.show_definition_btn, 0, 2, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.arg_layout.addItem(self.horizontalSpacer, 1, 1, 1, 1)

        self.label_9 = QLabel(self.args_widget)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setMinimumSize(QSize(155, 0))

        self.arg_layout.addWidget(self.label_9, 0, 0, 1, 1)

        self.functions = QComboBox(self.args_widget)
        self.functions.setObjectName(u"functions")

        self.arg_layout.addWidget(self.functions, 0, 1, 1, 1)

        self.label = QLabel(self.args_widget)
        self.label.setObjectName(u"label")

        self.arg_layout.addWidget(self.label, 1, 0, 1, 1)

        self.scrollArea.setWidget(self.args_widget)

        self.verticalLayout.addWidget(self.scrollArea)


        self.gridLayout.addWidget(self.groupBox, 2, 0, 1, 5)

        self.groupBox_2 = QGroupBox(ComputedChannel)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_2 = QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setHorizontalSpacing(2)
        self.gridLayout_2.setVerticalSpacing(12)
        self.gridLayout_2.setContentsMargins(4, 4, 4, 4)
        self.sample_by_sample = QRadioButton(self.groupBox_2)
        self.sample_by_sample.setObjectName(u"sample_by_sample")
        self.sample_by_sample.setChecked(True)

        self.gridLayout_2.addWidget(self.sample_by_sample, 0, 0, 1, 1)

        self.label_24 = QLabel(self.groupBox_2)
        self.label_24.setObjectName(u"label_24")
        self.label_24.setTextFormat(Qt.RichText)
        self.label_24.setWordWrap(False)

        self.gridLayout_2.addWidget(self.label_24, 0, 1, 1, 1)

        self.complete_signal = QRadioButton(self.groupBox_2)
        self.complete_signal.setObjectName(u"complete_signal")
        self.complete_signal.setChecked(False)

        self.gridLayout_2.addWidget(self.complete_signal, 1, 0, 1, 1)

        self.label_25 = QLabel(self.groupBox_2)
        self.label_25.setObjectName(u"label_25")

        self.gridLayout_2.addWidget(self.label_25, 1, 1, 1, 1)

        self.gridLayout_2.setColumnStretch(1, 1)

        self.gridLayout.addWidget(self.groupBox_2, 1, 0, 1, 5)

        self.gridLayout.setRowStretch(2, 1)
        QWidget.setTabOrder(self.triggering_on_all, self.triggering_on_interval)
        QWidget.setTabOrder(self.triggering_on_interval, self.trigger_interval)
        QWidget.setTabOrder(self.trigger_interval, self.triggering_on_channel)
        QWidget.setTabOrder(self.triggering_on_channel, self.trigger_channel)
        QWidget.setTabOrder(self.trigger_channel, self.trigger_search_btn)
        QWidget.setTabOrder(self.trigger_search_btn, self.sample_by_sample)
        QWidget.setTabOrder(self.sample_by_sample, self.complete_signal)
        QWidget.setTabOrder(self.complete_signal, self.scrollArea)
        QWidget.setTabOrder(self.scrollArea, self.functions)
        QWidget.setTabOrder(self.functions, self.show_definition_btn)
        QWidget.setTabOrder(self.show_definition_btn, self.name)
        QWidget.setTabOrder(self.name, self.comment)
        QWidget.setTabOrder(self.comment, self.unit)
        QWidget.setTabOrder(self.unit, self.apply_btn)
        QWidget.setTabOrder(self.apply_btn, self.cancel_btn)

        self.retranslateUi(ComputedChannel)

        QMetaObject.connectSlotsByName(ComputedChannel)
    # setupUi

    def retranslateUi(self, ComputedChannel):
        ComputedChannel.setWindowTitle(QCoreApplication.translate("ComputedChannel", u"Define new channel", None))
        self.apply_btn.setText(QCoreApplication.translate("ComputedChannel", u"Apply", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("ComputedChannel", u"Triggering", None))
        self.trigger_interval.setSuffix(QCoreApplication.translate("ComputedChannel", u"s", None))
        self.triggering_on_channel.setText(QCoreApplication.translate("ComputedChannel", u"on channel", None))
        self.triggering_on_interval.setText(QCoreApplication.translate("ComputedChannel", u"time interval", None))
        self.trigger_search_btn.setText("")
        self.triggering_on_all.setText(QCoreApplication.translate("ComputedChannel", u"all channels timestamps", None))
        self.cancel_btn.setText(QCoreApplication.translate("ComputedChannel", u"Cancel", None))
        self.label_6.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel unit", None))
        self.label_8.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel comment", None))
        self.unit.setPlaceholderText("")
        self.label_7.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel name", None))
        self.name.setInputMask("")
        self.name.setText("")
        self.name.setPlaceholderText("")
        self.groupBox.setTitle(QCoreApplication.translate("ComputedChannel", u"Function", None))
        self.show_definition_btn.setText("")
        self.label_9.setText(QCoreApplication.translate("ComputedChannel", u"Name", None))
        self.label.setText(QCoreApplication.translate("ComputedChannel", u"Arguments:", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("ComputedChannel", u"Computation mode", None))
        self.sample_by_sample.setText(QCoreApplication.translate("ComputedChannel", u"sample by sample", None))
        self.label_24.setText(QCoreApplication.translate("ComputedChannel", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"</style></head><body style=\" font-family:'Segoe UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">the function will be called several times for each of the individual time stamp in the time base</li>\n"
"<li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">earch argument will receive the signal sample at the"
                        " current time stamp</li>\n"
"<li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">the<span style=\" font-weight:700; font-style:italic;\"> t</span> argument will receive the current time stamp value</li></ul></body></html>", None))
        self.complete_signal.setText(QCoreApplication.translate("ComputedChannel", u"complete signal", None))
        self.label_25.setText(QCoreApplication.translate("ComputedChannel", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"</style></head><body style=\" font-family:'Segoe UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">the function will be called only once</li>\n"
"<li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">each argument will receive the complete signal ndarray</li></ul>\n"
"<ul style=\"margin-top: 0px; margin-bottom:"
                        " 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:'MS Shell Dlg 2'; font-size:8pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">the<span style=\" font-style:italic;\"> </span><span style=\" font-weight:700; font-style:italic;\">t</span><span style=\" font-style:italic;\"> </span>argument will receive the complete time base ndarray</li></ul></body></html>", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'signal_scale.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDialog, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QHBoxLayout,
    QLabel, QPushButton, QSizePolicy, QSpacerItem,
    QWidget)
from . import resource_rc

class Ui_ScaleDialog(object):
    def setupUi(self, ScaleDialog):
        if not ScaleDialog.objectName():
            ScaleDialog.setObjectName(u"ScaleDialog")
        ScaleDialog.resize(1091, 678)
        self.gridLayout_3 = QGridLayout(ScaleDialog)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_6)

        self.cancel_btn = QPushButton(ScaleDialog)
        self.cancel_btn.setObjectName(u"cancel_btn")
        icon = QIcon()
        icon.addFile(u":/erase.png", QSize(), QIcon.Normal, QIcon.Off)
        self.cancel_btn.setIcon(icon)

        self.horizontalLayout.addWidget(self.cancel_btn)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_7)

        self.apply_btn = QPushButton(ScaleDialog)
        self.apply_btn.setObjectName(u"apply_btn")
        icon1 = QIcon()
        icon1.addFile(u":/checkmark.png", QSize(), QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon1)

        self.horizontalLayout.addWidget(self.apply_btn)

        self.horizontalLayout.setStretch(0, 1)

        self.gridLayout_3.addLayout(self.horizontalLayout, 5, 0, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 135, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_3.addItem(self.verticalSpacer, 4, 1, 1, 1)

        self.groupBox_2 = QGroupBox(ScaleDialog)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout = QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName(u"gridLayout")
        self.fit_btn = QPushButton(self.groupBox_2)
        self.fit_btn.setObjectName(u"fit_btn")
        icon2 = QIcon()
        icon2.addFile(u":/fit.png", QSize(), QIcon.Normal, QIcon.Off)
        self.fit_btn.setIcon(icon2)

        self.gridLayout.addWidget(self.fit_btn, 2, 0, 1, 1)

        self.label_3 = QLabel(self.groupBox_2)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_3, 2, 1, 1, 1)

        self.zoom_out_btn = QPushButton(self.groupBox_2)
        self.zoom_out_btn.setObjectName(u"zoom_out_btn")
        icon3 = QIcon()
        icon3.addFile(u":/zoom-out.png", QSize(), QIcon.Normal, QIcon.Off)
        self.zoom_out_btn.setIcon(icon3)

        self.gridLayout.addWidget(self.zoom_out_btn, 1, 0, 1, 1)

        self.label = QLabel(self.groupBox_2)
        self.label.setObjectName(u"label")
        self.label.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label, 1, 1, 1, 1)

        self.fast_shift_up_btn = QPushButton(self.groupBox_2)
        self.fast_shift_up_btn.setObjectName(u"fast_shift_up_btn")
        icon4 = QIcon()
        icon4.addFile(u":/shift_up.png", QSize(), QIcon.Normal, QIcon.Off)
        self.fast_shift_up_btn.setIcon(icon4)

        self.gridLayout.addWidget(self.fast_shift_up_btn, 5, 0, 1, 1)

        self.label_2 = QLabel(self.groupBox_2)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_2, 0, 1, 1, 1)

        self.shift_down_btn = QPushButton(self.groupBox_2)
        self.shift_down_btn.setObjectName(u"shift_down_btn")
        icon5 = QIcon()
        icon5.addFile(u":/down.png", QSize(), QIcon.Normal, QIcon.Off)
        self.shift_down_btn.setIcon(icon5)

        self.gridLayout.addWidget(self.shift_down_btn, 4, 0, 1, 1)

        self.shift_up_btn = QPushButton(self.groupBox_2)
        self.shift_up_btn.setObjectName(u"shift_up_btn")
        icon6 = QIcon()
        icon6.addFile(u":/up.png", QSize(), QIcon.Normal, QIcon.Off)
        self.shift_up_btn.setIcon(icon6)

        self.gridLayout.addWidget(self.shift_up_btn, 3, 0, 1, 1)

        self.zoom_in_btn = QPushButton(self.groupBox_2)
        self.zoom_in_btn.setObjectName(u"zoom_in_btn")
        icon7 = QIcon()
        icon7.addFile(u":/zoom-in.png", QSize(), QIcon.Normal, QIcon.Off)
        self.zoom_in_btn.setIcon(icon7)

        self.gridLayout.addWidget(self.zoom_in_btn, 0, 0, 1, 1)

        self.fast_shift_down_btn = QPushButton(self.groupBox_2)
        self.fast_shift_down_btn.setObjectName(u"fast_shift_down_btn")
        icon8 = QIcon()
        icon8.addFile(u":/shift_down.png", QSize(), QIcon.Normal, QIcon.Off)
        self.fast_shift_down_btn.setIcon(icon8)

        self.gridLayout.addWidget(self.fast_shift_down_btn, 6, 0, 1, 1)


        self.gridLayout_3.addWidget(self.groupBox_2, 2, 1, 1, 1)

        self.plot = QLabel(ScaleDialog)
        self.plot.setObjectName(u"plot")
        self.plot.setMinimumSize(QSize(750, 600))
        self.plot.setMaximumSize(QSize(750, 600))
        self.plot.setFrameShape(QFrame.Box)
        self.plot.setLineWidth(2)

        self.gridLayout_3.addWidget(self.plot, 1, 0, 4, 1)

        self.groupBox_3 = QGroupBox(ScaleDialog)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.gridLayout_4 = QGridLayout(self.groupBox_3)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.label_5 = QLabel(self.groupBox_3)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_4.addWidget(self.label_5, 0, 0, 1, 1)

        self.target_max = QDoubleSpinBox(self.groupBox_3)
        self.target_max.setObjectName(u"target_max")
        self.target_max.setDecimals(3)
        self.target_max.setMinimum(0.000000000000000)
        self.target_max.setMaximum(1.000000000000000)

        self.gridLayout_4.addWidget(self.target_max, 0, 1, 1, 1)

        self.label_9 = QLabel(self.groupBox_3)
        self.label_9.setObjectName(u"label_9")

        self.gridLayout_4.addWidget(self.label_9, 1, 0, 1, 1)

        self.target_min = QDoubleSpinBox(self.groupBox_3)
        self.target_min.setObjectName(u"target_min")
        self.target_min.setDecimals(3)
        self.target_min.setMinimum(0.000000000000000)
        self.target_min.setMaximum(1.000000000000000)

        self.gridLayout_4.addWidget(self.target_min, 1, 1, 1, 1)


        self.gridLayout_3.addWidget(self.groupBox_3, 1, 1, 1, 1)

        self.groupBox_4 = QGroupBox(ScaleDialog)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.gridLayout_2 = QGridLayout(self.groupBox_4)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_10 = QLabel(self.groupBox_4)
        self.label_10.setObjectName(u"label_10")

        self.gridLayout_2.addWidget(self.label_10, 3, 0, 1, 1)

        self.label_8 = QLabel(self.groupBox_4)
        self.label_8.setObjectName(u"label_8")

        self.gridLayout_2.addWidget(self.label_8, 1, 0, 1, 1)

        self.offset = QDoubleSpinBox(self.groupBox_4)
        self.offset.setObjectName(u"offset")
        self.offset.setDecimals(1)
        self.offset.setMinimum(0.000000000000000)
        self.offset.setMaximum(1.000000000000000)

        self.gridLayout_2.addWidget(self.offset, 3, 1, 1, 1)

        self.label_4 = QLabel(self.groupBox_4)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_2.addWidget(self.label_4, 0, 0, 1, 1)

        self.label_13 = QLabel(self.groupBox_4)
        self.label_13.setObjectName(u"label_13")

        self.gridLayout_2.addWidget(self.label_13, 2, 0, 1, 1)

        self.scaling = QDoubleSpinBox(self.groupBox_4)
        self.scaling.setObjectName(u"scaling")
        self.scaling.setDecimals(6)

        self.gridLayout_2.addWidget(self.scaling, 2, 1, 1, 1)

        self.y_top = QLabel(self.groupBox_4)
        self.y_top.setObjectName(u"y_top")

        self.gridLayout_2.addWidget(self.y_top, 0, 1, 1, 1)

        self.y_bottom = QLabel(self.groupBox_4)
        self.y_bottom.setObjectName(u"y_bottom")

        self.gridLayout_2.addWidget(self.y_bottom, 1, 1, 1, 1)


        self.gridLayout_3.addWidget(self.groupBox_4, 3, 1, 1, 1)

        self.signal = QComboBox(ScaleDialog)
        self.signal.setObjectName(u"signal")

        self.gridLayout_3.addWidget(self.signal, 0, 0, 1, 2)

        QWidget.setTabOrder(self.signal, self.target_max)
        QWidget.setTabOrder(self.target_max, self.target_min)
        QWidget.setTabOrder(self.target_min, self.zoom_in_btn)
        QWidget.setTabOrder(self.zoom_in_btn, self.zoom_out_btn)
        QWidget.setTabOrder(self.zoom_out_btn, self.fit_btn)
        QWidget.setTabOrder(self.fit_btn, self.shift_up_btn)
        QWidget.setTabOrder(self.shift_up_btn, self.shift_down_btn)
        QWidget.setTabOrder(self.shift_down_btn, self.fast_shift_up_btn)
        QWidget.setTabOrder(self.fast_shift_up_btn, self.fast_shift_down_btn)
        QWidget.setTabOrder(self.fast_shift_down_btn, self.scaling)
        QWidget.setTabOrder(self.scaling, self.offset)
        QWidget.setTabOrder(self.offset, self.apply_btn)
        QWidget.setTabOrder(self.apply_btn, self.cancel_btn)

        self.retranslateUi(ScaleDialog)

        QMetaObject.connectSlotsByName(ScaleDialog)
    # setupUi

    def retranslateUi(self, ScaleDialog):
        ScaleDialog.setWindowTitle(QCoreApplication.translate("ScaleDialog", u"Dialog", None))
        self.cancel_btn.setText(QCoreApplication.translate("ScaleDialog", u"Cancel", None))
        self.apply_btn.setText(QCoreApplication.translate("ScaleDialog", u"Apply", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("ScaleDialog", u"Keyboard shortcuts", None))
        self.fit_btn.setText(QCoreApplication.translate("ScaleDialog", u"Fit vertically", None))
        self.label_3.setText(QCoreApplication.translate("ScaleDialog", u"F", None))
        self.zoom_out_btn.setText(QCoreApplication.translate("ScaleDialog", u"Zoom out", None))
        self.label.setText(QCoreApplication.translate("ScaleDialog", u"O", None))
        self.fast_shift_up_btn.setText(QCoreApplication.translate("ScaleDialog", u"Fast shift up", None))
        self.label_2.setText(QCoreApplication.translate("ScaleDialog", u"I", None))
        self.shift_down_btn.setText(QCoreApplication.translate("ScaleDialog", u"Shift down", None))
        self.shift_up_btn.setText(QCoreApplication.translate("ScaleDialog", u"Shift up", None))
        self.zoom_in_btn.setText(QCoreApplication.translate("ScaleDialog", u"Zoom in", None))
        self.fast_shift_down_btn.setText(QCoreApplication.translate("ScaleDialog", u"Fast shift down", None))
        self.plot.setText("")
        self.groupBox_3.setTitle(QCoreApplication.translate("ScaleDialog", u"Expected signal values", None))
        self.label_5.setText(QCoreApplication.translate("ScaleDialog", u"Signal max", None))
        self.label_9.setText(QCoreApplication.translate("ScaleDialog", u"Signal min", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("ScaleDialog", u"Screen levels", None))
        self.label_10.setText(QCoreApplication.translate("ScaleDialog", u"0 level on screen", None))
        self.label_8.setText(QCoreApplication.translate("ScaleDialog", u"Y axis bottom value", None))
        self.offset.setSuffix(QCoreApplication.translate("ScaleDialog", u"%", None))
        self.label_4.setText(QCoreApplication.translate("ScaleDialog", u"Y axis top value", None))
        self.label_13.setText(QCoreApplication.translate("ScaleDialog", u"Scaling [units/screen]", None))
        self.y_top.setText("")
        self.y_bottom.setText("")
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'simple_search_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QComboBox, QDialog,
    QGridLayout, QHBoxLayout, QHeaderView, QLabel,
    QLineEdit, QPushButton, QSizePolicy, QSpacerItem,
    QTreeWidget, QTreeWidgetItem, QWidget)
import resource_rc

class Ui_SimpleSearchDialog(object):
    def setupUi(self, SimpleSearchDialog):
        if not SimpleSearchDialog.objectName():
            SimpleSearchDialog.setObjectName(u"SimpleSearchDialog")
        SimpleSearchDialog.resize(1183, 421)
        SimpleSearchDialog.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(SimpleSearchDialog)
        self.gridLayout.setObjectName(u"gridLayout")
        self.matches = QTreeWidget(SimpleSearchDialog)
        self.matches.setObjectName(u"matches")
        self.matches.setFocusPolicy(Qt.TabFocus)
        self.matches.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.matches.setSortingEnabled(True)

        self.gridLayout.addWidget(self.matches, 3, 0, 2, 1)

        self.status = QLabel(SimpleSearchDialog)
        self.status.setObjectName(u"status")

        self.gridLayout.addWidget(self.status, 9, 0, 1, 1)

        self.selection = QTreeWidget(SimpleSearchDialog)
        self.selection.setObjectName(u"selection")
        self.selection.setSortingEnabled(True)

        self.gridLayout.addWidget(self.selection, 3, 2, 2, 1)

        self.label = QLabel(SimpleSearchDialog)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 2, 2, 1, 1)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)


        self.gridLayout.addLayout(self.horizontalLayout_2, 0, 2, 1, 1)

        self.add_btn = QPushButton(SimpleSearchDialog)
        self.add_btn.setObjectName(u"add_btn")
        icon = QIcon()
        icon.addFile(u":/left.png", QSize(), QIcon.Normal, QIcon.Off)
        self.add_btn.setIcon(icon)

        self.gridLayout.addWidget(self.add_btn, 3, 1, 1, 1)

        self.search_box = QLineEdit(SimpleSearchDialog)
        self.search_box.setObjectName(u"search_box")

        self.gridLayout.addWidget(self.search_box, 2, 0, 1, 1)

        self.match_kind = QComboBox(SimpleSearchDialog)
        self.match_kind.addItem("")
        self.match_kind.addItem("")
        self.match_kind.setObjectName(u"match_kind")

        self.gridLayout.addWidget(self.match_kind, 0, 0, 1, 1)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer_2, 4, 1, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.cancel_btn = QPushButton(SimpleSearchDialog)
        self.cancel_btn.setObjectName(u"cancel_btn")

        self.horizontalLayout.addWidget(self.cancel_btn)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.apply_btn = QPushButton(SimpleSearchDialog)
        self.apply_btn.setObjectName(u"apply_btn")

        self.horizontalLayout.addWidget(self.apply_btn)

        self.horizontalLayout.setStretch(1, 1)

        self.gridLayout.addLayout(self.horizontalLayout, 6, 0, 1, 3)

        self.comment = QLabel(SimpleSearchDialog)
        self.comment.setObjectName(u"comment")

        self.gridLayout.addWidget(self.comment, 5, 0, 1, 3)

        self.case_sensitivity = QComboBox(SimpleSearchDialog)
        self.case_sensitivity.addItem("")
        self.case_sensitivity.addItem("")
        self.case_sensitivity.setObjectName(u"case_sensitivity")

        self.gridLayout.addWidget(self.case_sensitivity, 1, 0, 1, 1)

        QWidget.setTabOrder(self.match_kind, self.case_sensitivity)
        QWidget.setTabOrder(self.case_sensitivity, self.search_box)
        QWidget.setTabOrder(self.search_box, self.matches)
        QWidget.setTabOrder(self.matches, self.add_btn)
        QWidget.setTabOrder(self.add_btn, self.selection)
        QWidget.setTabOrder(self.selection, self.apply_btn)
        QWidget.setTabOrder(self.apply_btn, self.cancel_btn)

        self.retranslateUi(SimpleSearchDialog)

        self.match_kind.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(SimpleSearchDialog)
    # setupUi

    def retranslateUi(self, SimpleSearchDialog):
        SimpleSearchDialog.setWindowTitle(QCoreApplication.translate("SimpleSearchDialog", u"Dialog", None))
        ___qtreewidgetitem = self.matches.headerItem()
        ___qtreewidgetitem.setText(0, QCoreApplication.translate("SimpleSearchDialog", u"Channel", None));
        self.status.setText("")
        ___qtreewidgetitem1 = self.selection.headerItem()
        ___qtreewidgetitem1.setText(0, QCoreApplication.translate("SimpleSearchDialog", u"Channel", None));
        self.label.setText(QCoreApplication.translate("SimpleSearchDialog", u"Final selection", None))
        self.add_btn.setText("")
        self.search_box.setText("")
        self.match_kind.setItemText(0, QCoreApplication.translate("SimpleSearchDialog", u"Wildcard", None))
        self.match_kind.setItemText(1, QCoreApplication.translate("SimpleSearchDialog", u"Regex", None))

        self.cancel_btn.setText(QCoreApplication.translate("SimpleSearchDialog", u"Cancel", None))
        self.apply_btn.setText(QCoreApplication.translate("SimpleSearchDialog", u"Apply", None))
        self.comment.setText("")
        self.case_sensitivity.setItemText(0, QCoreApplication.translate("SimpleSearchDialog", u"Case insensitive", None))
        self.case_sensitivity.setItemText(1, QCoreApplication.translate("SimpleSearchDialog", u"Case sensitive", None))

    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'vtt_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDoubleSpinBox, QHBoxLayout,
    QLabel, QLineEdit, QPushButton, QSizePolicy,
    QStackedWidget, QWidget)

class Ui_VTT_Widget(object):
    def setupUi(self, VTT_Widget):
        if not VTT_Widget.objectName():
            VTT_Widget.setObjectName(u"VTT_Widget")
        VTT_Widget.resize(511, 26)
        self.horizontalLayout = QHBoxLayout(VTT_Widget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(-1, 2, -1, 2)
        self.value = QDoubleSpinBox(VTT_Widget)
        self.value.setObjectName(u"value")
        self.value.setMinimumSize(QSize(100, 0))
        self.value.setDecimals(0)

        self.horizontalLayout.addWidget(self.value)

        self.label_2 = QLabel(VTT_Widget)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout.addWidget(self.label_2)

        self.mode_switch = QComboBox(VTT_Widget)
        self.mode_switch.addItem("")
        self.mode_switch.addItem("")
        self.mode_switch.setObjectName(u"mode_switch")

        self.horizontalLayout.addWidget(self.mode_switch)

        self.mode = QStackedWidget(VTT_Widget)
        self.mode.setObjectName(u"mode")
        self.page = QWidget()
        self.page.setObjectName(u"page")
        self.horizontalLayout_2 = QHBoxLayout(self.page)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.text = QLineEdit(self.page)
        self.text.setObjectName(u"text")

        self.horizontalLayout_2.addWidget(self.text)

        self.mode.addWidget(self.page)
        self.page_2 = QWidget()
        self.page_2.setObjectName(u"page_2")
        self.horizontalLayout_3 = QHBoxLayout(self.page_2)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.conversion_btn = QPushButton(self.page_2)
        self.conversion_btn.setObjectName(u"conversion_btn")

        self.horizontalLayout_3.addWidget(self.conversion_btn)

        self.mode.addWidget(self.page_2)

        self.horizontalLayout.addWidget(self.mode)


        self.retranslateUi(VTT_Widget)

        self.mode.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(VTT_Widget)
    # setupUi

    def retranslateUi(self, VTT_Widget):
        VTT_Widget.setWindowTitle(QCoreApplication.translate("VTT_Widget", u"Form", None))
        self.label_2.setText(QCoreApplication.translate("VTT_Widget", u"->", None))
        self.mode_switch.setItemText(0, QCoreApplication.translate("VTT_Widget", u"Text", None))
        self.mode_switch.setItemText(1, QCoreApplication.translate("VTT_Widget", u"Conversion", None))

        self.text.setPlaceholderText("")
        self.conversion_btn.setText(QCoreApplication.translate("VTT_Widget", u"Referenced conversion", None))
    # retranslateUi




import numpy as np
from PySide6 import QtCore, QtGui, QtWidgets
import scipy.signal as scipy_signal

from ..ui import resource_rc
from ..ui.fft_window import Ui_FFTWindow
from .plot import Plot, PlotSignal, Signal


class FFTWindow(Ui_FFTWindow, QtWidgets.QMainWindow):
    def __init__(self, signal, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self._settings = QtCore.QSettings()
        self.with_dots = self._settings.value("dots", False, type=bool)

        self.signal = signal

        self.signal_plot = Plot([], self.with_dots)
        self.fft_plot = Plot([], self.with_dots, x_axis="frequency")

        layout = self.layout
        layout.addWidget(self.signal_plot)
        layout.addWidget(self.fft_plot)

        self.show()
        self.signal_plot.add_new_channels([signal])

        self.signal_plot.region_removed_signal.connect(self.update)
        self.signal_plot.region_moved_signal.connect(self.update)
        self.start_frequency.valueChanged.connect(self.update)
        self.end_frequency.valueChanged.connect(self.update)
        self.frequency_step.valueChanged.connect(self.update)

        self.setWindowTitle(f"{self.signal.name} FFT using Lomb-Scargle periodogram")

        self.update(initial=True)

    def update(self, *args, initial=False):
        xrange, yrange = self.fft_plot.plot.viewbox.viewRange()

        if self.signal_plot.plot.region:
            start, stop = self.signal_plot.plot.region.getRegion()
            signal = self.signal.cut(start, stop)
        else:
            signal = self.signal

        start_frequency = self.start_frequency.value()
        end_frequency = self.end_frequency.value()
        frequency_step = self.frequency_step.value()
        if start_frequency > end_frequency:
            start_frequency, end_frequency = end_frequency, start_frequency
        steps = int((end_frequency - start_frequency) / frequency_step)

        self.fft_plot.clear()

        if len(self.signal) and steps:
            f = np.linspace(start_frequency, end_frequency, steps)
            pgram = scipy_signal.lombscargle(signal.timestamps, signal.samples, f * 2 * np.pi, normalize=True)

            signal = Signal(samples=pgram, timestamps=f, name=f"{self.signal.name}_FFT")
            signal.color = self.signal.color
            signal.computed = False
            signal.computation = {}

            signal = PlotSignal(signal)

            self.fft_plot.add_new_channels([signal])

        if not initial:
            self.fft_plot.plot.viewbox.setXRange(*xrange, padding=0)
            self.fft_plot.plot.viewbox.setYRange(*yrange, padding=0)



from PySide6 import QtCore, QtWidgets


class SearchTreeItem(QtWidgets.QTreeWidgetItem):
    def __init__(self, name, group, index, unit, source_name, source_path, comment):
        super().__init__([name, group, index, unit, source_name, source_path, comment])

        self.name = name
        self.group = group
        self.index = index
        self.unit = unit
        self.source_name = source_name
        self.source_path = source_path
        self.comment = comment

    def __del__(self):
        self.name = None
        self.group = None
        self.index = None
        self.unit = None
        self.source_name = None
        self.source_path = None
        self.comment = None


class SearchTreeWidget(QtWidgets.QTreeWidget):
    def __init__(self, can_delete_items=False, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
        self.setDragDropMode(QtWidgets.QAbstractItemView.DragDropMode.NoDragDrop)
        self.setUniformRowHeights(True)

        self.can_delete_items = can_delete_items

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key.Key_Delete and self.can_delete_items:
            selected_items = self.selectedItems()

            root = self.invisibleRootItem()
            for item in selected_items:
                (item.parent() or root).removeChild(item)



from PySide6 import QtCore, QtWidgets

from ..ui import resource_rc
from ..ui.range_editor_dialog import Ui_RangeDialog
from ..widgets.range_widget import RangeWidget


class RangeEditor(Ui_RangeDialog, QtWidgets.QDialog):
    def __init__(self, name, unit="", ranges=None, brush=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)
        self.setWindowFlags(
            self.windowFlags()
            | QtCore.Qt.WindowType.WindowSystemMenuHint
            | QtCore.Qt.WindowType.WindowMinMaxButtonsHint
        )

        self.name = name
        self.unit = unit
        self.result = []
        self.pressed_button = None
        self._brush = brush

        if ranges:
            for range_info in ranges:
                widget = RangeWidget(
                    parent=self,
                    name=self.name,
                    brush=brush,
                    **range_info,
                )

                item = QtWidgets.QListWidgetItem()
                item.setSizeHint(widget.sizeHint())
                self.ranges.addItem(item)
                self.ranges.setItemWidget(item, widget)

        self.ranges.setUniformItemSizes(True)

        self.apply_btn.clicked.connect(self.apply)
        self.insert_btn.clicked.connect(self.insert)
        self.cancel_btn.clicked.connect(self.cancel)
        self.reset_btn.clicked.connect(self.reset)

        self.setWindowTitle(f"Edit {self.name} range colors")
        self.show()

    def apply(self, event):
        ranges = []
        count = self.ranges.count()
        for i in range(count):
            item = self.ranges.item(i)
            if item is None:
                continue

            widget = self.ranges.itemWidget(item)
            ranges.append(widget.to_dict(brush=self._brush))

        self.result = ranges
        self.pressed_button = "apply"
        self.close()

    def insert(self, event):
        widget = RangeWidget(self.name)

        item = QtWidgets.QListWidgetItem()
        item.setSizeHint(widget.sizeHint())
        self.ranges.addItem(item)
        self.ranges.setItemWidget(item, widget)

    def reset(self, event):
        self.ranges.clear()
        self.result = []

    def cancel(self, event):
        self.result = []
        self.pressed_button = "cancel"
        self.close()



#!/usr/bin/env python
import unittest

import numpy as np

from asammdf import Signal
from asammdf.blocks.utils import MdfException


class TestSignal(unittest.TestCase):
    def test_init_empty(self):
        s = Signal([], [], name="s")
        self.assertEqual(len(s), 0)

    def test_init_errors(self):
        with self.assertRaises(MdfException):
            Signal([], [1], name="s")
        with self.assertRaises(MdfException):
            Signal([1], [], name="s")
        with self.assertRaises(MdfException):
            Signal([1, 2, 3], [1, 2], name="s")
        with self.assertRaises(MdfException):
            Signal(np.array([]), np.array([1]), name="s")
        with self.assertRaises(MdfException):
            Signal(np.array([1]), np.array([]), name="s")
        with self.assertRaises(MdfException):
            Signal(np.array([1, 2, 3]), np.array([1, 2]), name="s")
        with self.assertRaises(MdfException):
            Signal(np.array([]), np.array([]))

    def test_cut_int(self):
        s = Signal(np.arange(5), np.arange(5, dtype="<f8"), name="S")

        res = s.cut()
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        # stop == None

        res = s.cut(start=-1)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=-1, include_ends=False)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0, include_ends=False)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0.5)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0.5, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.5, include_ends=False)
        self.assertTrue(np.array_equal([1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        # start == None

        res = s.cut(stop=-1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(stop=-1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0.5)
        self.assertTrue(np.array_equal([0, 0], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 0.5], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0.5, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4, include_ends=False)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4.1)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4.1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        # with start and stop

        res = s.cut(start=-2, stop=-1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=-1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0.5)
        self.assertTrue(np.array_equal([0, 0], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 0.5], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0.5, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=1.1)
        self.assertTrue(np.array_equal([0, 1, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 1.1], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=1.1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0, stop=1)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0, stop=1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.1, stop=3.5)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 3], res.samples))
        self.assertTrue(np.array_equal(np.array([0.1, 1, 2, 3, 3.5], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.1, stop=3.5, include_ends=False)
        self.assertTrue(np.array_equal([1, 2, 3], res.samples))
        self.assertTrue(np.array_equal(np.array([1, 2, 3], dtype="<f8"), res.timestamps))

        res = s.cut(start=1.1, stop=1.9)
        self.assertTrue(np.array_equal([1, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([1.1, 1.9], dtype="<f8"), res.timestamps))

        res = s.cut(start=1.1, stop=1.9, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=3.5, stop=4.5)
        self.assertTrue(np.array_equal([3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([3.5, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=3.5, stop=4.5, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, stop=4.5)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, stop=4.5, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, stop=4.2)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, stop=4.2, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

    def test_cut_float(self):
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        res = s.cut()
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        # stop == None

        res = s.cut(start=-1)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=-1, include_ends=False)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0, include_ends=False)
        self.assertTrue(np.array_equal(np.arange(5), res.samples))
        self.assertTrue(np.array_equal(np.arange(5, dtype="<f8"), res.timestamps))

        res = s.cut(start=0.5)
        self.assertTrue(np.allclose([0.5, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0.5, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.5, include_ends=False)
        self.assertTrue(np.array_equal([1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        # start == None

        res = s.cut(stop=-1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(stop=-1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0.5)
        self.assertTrue(np.allclose([0, 0.5], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 0.5], dtype="<f8"), res.timestamps))

        res = s.cut(stop=0.5, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4, include_ends=False)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4.1)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        res = s.cut(stop=4.1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1, 2, 3, 4], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 2, 3, 4], dtype="<f8"), res.timestamps))

        # with start and stop

        res = s.cut(start=-2, stop=-1)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=-1, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0.5)
        self.assertTrue(np.allclose([0, 0.5], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 0.5], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=0.5, include_ends=False)
        self.assertTrue(np.array_equal([0], res.samples))
        self.assertTrue(np.array_equal(np.array([0], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=1.1)
        self.assertTrue(np.allclose([0, 1, 1.1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1, 1.1], dtype="<f8"), res.timestamps))

        res = s.cut(start=-2, stop=1.1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0, stop=1)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0, stop=1, include_ends=False)
        self.assertTrue(np.array_equal([0, 1], res.samples))
        self.assertTrue(np.array_equal(np.array([0, 1], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.1, stop=3.5)
        self.assertTrue(np.allclose([0.1, 1, 2, 3, 3.5], res.samples))
        self.assertTrue(np.allclose(np.array([0.1, 1, 2, 3, 3.5], dtype="<f8"), res.timestamps))

        res = s.cut(start=0.1, stop=3.5, include_ends=False)
        self.assertTrue(np.array_equal([1, 2, 3], res.samples))
        self.assertTrue(np.array_equal(np.array([1, 2, 3], dtype="<f8"), res.timestamps))

        res = s.cut(start=1.1, stop=1.9)
        self.assertTrue(np.allclose([1.1, 1.9], res.samples))
        self.assertTrue(np.allclose(np.array([1.1, 1.9], dtype="<f8"), res.timestamps))

        res = s.cut(start=1.1, stop=1.9, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=3.5, stop=4.5)
        self.assertTrue(np.allclose([3.5, 4], res.samples))
        self.assertTrue(np.allclose(np.array([3.5, 4], dtype="<f8"), res.timestamps))

        res = s.cut(start=3.5, stop=4.5, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, stop=4.5)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4, stop=4.5, include_ends=False)
        self.assertTrue(np.array_equal([4], res.samples))
        self.assertTrue(np.array_equal(np.array([4], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, stop=4.2)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

        res = s.cut(start=4.1, stop=4.2, include_ends=False)
        self.assertTrue(np.array_equal([], res.samples))
        self.assertTrue(np.array_equal(np.array([], dtype="<f8"), res.timestamps))

    def test_add(self):
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        target = np.arange(0, 10, 2, dtype="<f4")

        res = s + s
        self.assertTrue(np.array_equal(res.samples, target))

        s += s
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

        # + 2
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        target = np.arange(2, 7, dtype="<f4")

        res = s + 2
        self.assertTrue(np.array_equal(res.samples, target))

        res = 2 + s
        self.assertTrue(np.array_equal(res.samples, target))

        s += 2
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

    def test_sub(self):
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        target = np.zeros(5, dtype="<f4")

        res = s - s
        self.assertTrue(np.array_equal(res.samples, target))

        s -= s
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

        # - 2
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        target = np.arange(-2, 3, dtype="<f4")

        res = s - 2
        self.assertTrue(np.array_equal(res.samples, target))

        res = -(2 - s)
        self.assertTrue(np.array_equal(res.samples, target))

        s -= 2
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

    def test_mul(self):
        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")

        target = np.arange(5, dtype="<f4") ** 2

        res = s * s
        self.assertTrue(np.array_equal(res.samples, target))

        s *= s
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

        s = Signal(np.arange(5, dtype="<f4"), np.arange(5, dtype="<f8"), name="S")
        target = np.arange(0, 10, 2, dtype="<f4")

        res = s * 2
        self.assertTrue(np.array_equal(res.samples, target))

        res = 2 * s
        self.assertTrue(np.array_equal(res.samples, target))

        s *= 2
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

    def test_div(self):
        s = Signal(np.arange(1, 5, dtype="<f4"), np.arange(1, 5, dtype="<f8"), name="S")

        target = np.ones(4, dtype="<f4")

        res = s / s
        self.assertTrue(np.array_equal(res.samples, target))

        s /= s
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

        s = Signal(np.arange(1, 5, dtype="<f4"), np.arange(1, 5, dtype="<f8"), name="S")
        target = np.arange(2, 10, 2, dtype="<f4")

        res = 1 / (0.5 / s)
        self.assertTrue(np.array_equal(res.samples, target))

        s /= 0.5
        res = s
        self.assertTrue(np.array_equal(res.samples, target))

    def test_pow(self):
        s = Signal(np.arange(1, 5, dtype="<f4"), np.arange(1, 5, dtype="<f8"), name="S")

        target = np.arange(1, 5, dtype="<f4") ** 3

        res = s**3
        self.assertTrue(np.array_equal(res.samples, target))


if __name__ == "__main__":
    unittest.main()



import re

from natsort import natsorted
from PySide6 import QtCore, QtWidgets

from ..ui import resource_rc
from ..ui.gps_dialog import Ui_GPSDialog
from .advanced_search import AdvancedSearch


class GPSDialog(Ui_GPSDialog, QtWidgets.QDialog):
    def __init__(
        self,
        mdf,
        latitude="",
        longitude="",
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self.mdf = mdf

        self.latitude.setText(latitude)
        self.longitude.setText(longitude)

        self.apply_btn.clicked.connect(self._apply)
        self.cancel_btn.clicked.connect(self._cancel)

        self.search_latitude_btn.clicked.connect(self.search_latitude)
        self.search_longitude_btn.clicked.connect(self.search_longitude)

        self.valid = False

    def _apply(self, event):
        if self.latitude.text().strip() and self.longitude.text().strip():
            self.valid = True

        self.close()

    def _cancel(self, event):
        self.close()

    def search_latitude(self, *args):
        dlg = AdvancedSearch(
            self.mdf,
            show_add_window=False,
            show_apply=True,
            show_pattern=False,
            window_title="Search for the latitude channel",
            return_names=True,
            parent=self,
        )
        dlg.setModal(True)
        dlg.exec_()
        result, pattern_window = dlg.result, dlg.pattern_window

        if result:
            self.latitude.setText(list(result)[0])

    def search_longitude(self, *args):
        dlg = AdvancedSearch(
            self.mdf,
            show_add_window=False,
            show_apply=True,
            show_pattern=False,
            window_title="Search for the longitude channel",
            return_names=True,
            parent=self,
        )
        dlg.setModal(True)
        dlg.exec_()
        result, pattern_window = dlg.result, dlg.pattern_window

        if result:
            self.longitude.setText(list(result)[0])



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'bus_database_manager.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QListWidgetItem, QPushButton,
    QSizePolicy, QSpacerItem, QTabWidget, QWidget)

from asammdf.gui.widgets.list import MinimalListWidget
from . import resource_rc

class Ui_BusDatabaseManager(object):
    def setupUi(self, BusDatabaseManager):
        if not BusDatabaseManager.objectName():
            BusDatabaseManager.setObjectName(u"BusDatabaseManager")
        BusDatabaseManager.resize(632, 267)
        self.gridLayout = QGridLayout(BusDatabaseManager)
        self.gridLayout.setObjectName(u"gridLayout")
        self.tabWidget = QTabWidget(BusDatabaseManager)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.gridLayout_4 = QGridLayout(self.tab)
        self.gridLayout_4.setSpacing(2)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setContentsMargins(2, 2, 2, 2)
        self.load_can_database_btn = QPushButton(self.tab)
        self.load_can_database_btn.setObjectName(u"load_can_database_btn")
        icon = QIcon()
        icon.addFile(u":/open.png", QSize(), QIcon.Normal, QIcon.Off)
        self.load_can_database_btn.setIcon(icon)

        self.gridLayout_4.addWidget(self.load_can_database_btn, 0, 0, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_4.addItem(self.horizontalSpacer, 0, 1, 1, 1)

        self.can_database_list = MinimalListWidget(self.tab)
        self.can_database_list.setObjectName(u"can_database_list")

        self.gridLayout_4.addWidget(self.can_database_list, 1, 0, 1, 2)

        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.gridLayout_8 = QGridLayout(self.tab_2)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_8.setContentsMargins(2, 2, 2, 2)
        self.load_lin_database_btn = QPushButton(self.tab_2)
        self.load_lin_database_btn.setObjectName(u"load_lin_database_btn")
        self.load_lin_database_btn.setIcon(icon)

        self.gridLayout_8.addWidget(self.load_lin_database_btn, 0, 0, 1, 1)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_8.addItem(self.horizontalSpacer_3, 0, 1, 1, 1)

        self.lin_database_list = MinimalListWidget(self.tab_2)
        self.lin_database_list.setObjectName(u"lin_database_list")

        self.gridLayout_8.addWidget(self.lin_database_list, 1, 0, 1, 2)

        self.tabWidget.addTab(self.tab_2, "")

        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)


        self.retranslateUi(BusDatabaseManager)

        self.tabWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(BusDatabaseManager)
    # setupUi

    def retranslateUi(self, BusDatabaseManager):
        BusDatabaseManager.setWindowTitle(QCoreApplication.translate("BusDatabaseManager", u"Form", None))
        self.load_can_database_btn.setText(QCoreApplication.translate("BusDatabaseManager", u"Load CAN database", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), QCoreApplication.translate("BusDatabaseManager", u"CAN", None))
        self.load_lin_database_btn.setText(QCoreApplication.translate("BusDatabaseManager", u"Load LIN database", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), QCoreApplication.translate("BusDatabaseManager", u"LIN", None))
    # retranslateUi




"""
asammdf

"""
from pathlib import Path

from numpy import get_include
from setuptools import Extension, find_packages, setup

PROJECT_PATH = Path(__file__).parent


with (PROJECT_PATH / "requirements.txt").open() as f:
    install_requires = [l.strip() for l in f.readlines()]


def _get_version():
    with PROJECT_PATH.joinpath("src", "asammdf", "version.py").open() as f:
        line = next(line for line in f if line.startswith("__version__"))

    version = line.partition("=")[2].strip()[1:-1]

    return version


def _get_long_description():
    description = PROJECT_PATH.joinpath("README.md").read_text(encoding="utf-8")

    return description


def _get_ext_modules():
    modules = [
        Extension(
            "asammdf.blocks.cutils",
            ["src/asammdf/blocks/cutils.c"],
            include_dirs=[get_include()],
            extra_compile_args=["-std=c99"],
        )
    ]

    return modules


setup(
    name="asammdf",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=_get_version(),
    description="ASAM MDF measurement data file parser",
    long_description=_get_long_description(),
    long_description_content_type=r"text/markdown",
    # The project's main homepage.
    url="https://github.com/danielhrisca/asammdf",
    # Author details
    author="Daniel Hrisca",
    author_email="daniel.hrisca@gmail.com",
    # Choose your license
    license="LGPLv3+",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 5 - Production/Stable",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: Scientific/Engineering",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    # Supported python versions
    python_requires=">=3.8",
    # What does your project relate to?
    keywords="read reader edit editor parse parser asam mdf measurement",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages("src"),
    package_dir={"": "src"},
    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=install_requires,
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        "decode": ["cChardet==2.1.5", "chardet"],
        "export": [
            "fastparquet",
            "h5py",
            "hdf5storage>=0.1.19",
            "python-snappy",
        ],
        "export_matlab_v5": "scipy",
        "gui": [
            "lxml>=4.9.2",
            "natsort",
            "psutil",
            "PySide6==6.6.0",
            "pyqtgraph==0.13.3",
            "pyqtlet2==0.9.3",
            "packaging",
            "QtPy==2.3.1",
        ],
        "encryption": ["cryptography", "keyring"],
        "symbolic_math": "sympy",
        "filesystem": "fsspec",
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={"asammdf.gui.ui": ["*.ui"]},
    include_package_data=True,
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    #    data_files=[('my_data', ['data/data_file'])],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={"console_scripts": ["asammdf = asammdf.gui.asammdfgui:main [gui,export,decode]"]},
    ext_modules=_get_ext_modules(),
)



from PySide6 import QtCore, QtGui, QtWidgets

DEFAULT_TIMEOUT = 60


class MessageBox(QtWidgets.QMessageBox):
    def __init__(self, *args, **kwargs):
        self.timeout = kwargs.pop("timeout", DEFAULT_TIMEOUT)
        informative_text = kwargs.pop("informative_text", "")
        detailed_text = kwargs.pop("detailed_text", "")
        escapeButton = kwargs.pop("escapeButton", None)
        defaultButton = kwargs.pop("defaultButton", None)

        super().__init__(*args, **kwargs)

        self.original_text = self.text()

        if defaultButton is not None:
            self.setDefaultButton(defaultButton)

        if self.defaultButton() is not None:
            self.setText(
                f"{self.original_text}\n\nThis message will be closed in {self.timeout}s\n"
                f'Default button - [{self.defaultButton().text().strip("&")}]\n'
                "Abort the countdown - [F1]"
            )
        else:
            self.setText(
                f"{self.original_text}\n\nThis message will be closed in {self.timeout}s\n" "Abort the countdown - [F1]"
            )

        if informative_text:
            self.setInformativeText(informative_text)

        if detailed_text:
            self.setDetailedText(detailed_text)

        if escapeButton is not None:
            self.setEscapeButton(escapeButton)

        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.tick)
        self.timer.start(1000)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key.Key_F1:
            self.timer.stop()
            self.setText(self.original_text)
        else:
            super().keyPressEvent(event)

    def tick(self):
        self.timeout -= 1
        default = self.defaultButton()

        if self.timeout <= 0:
            self.timer.stop()
            if default is not None:
                default.animateClick()
            else:
                self.done(0)
        else:
            if default is not None:
                self.setText(
                    f"{self.original_text}\n\nThis message will be closed in {self.timeout}s\n"
                    f'Default button - [{self.defaultButton().text().strip("&")}]\n'
                    "Abort the countdown - [F1]"
                )
            else:
                self.setText(
                    f"{self.original_text}\n\nThis message will be closed in {self.timeout}s\n"
                    "Abort the countdown - [F1]"
                )

    @classmethod
    def about(
        cls,
        parent,
        title,
        text,
        buttons=QtWidgets.QMessageBox.StandardButton.Ok,
        defaultButton=QtWidgets.QMessageBox.StandardButton.Ok,
        escapeButton=QtWidgets.QMessageBox.StandardButton.Ok,
        timeout=DEFAULT_TIMEOUT,
    ):
        msg = cls(
            QtWidgets.QMessageBox.Icon.NoIcon,
            title,
            text,
            buttons,
            parent,
            timeout=timeout,
            defaultButton=defaultButton,
            escapeButton=escapeButton,
        )
        return msg.exec()

    @classmethod
    def critical(
        cls,
        parent,
        title,
        text,
        buttons=QtWidgets.QMessageBox.StandardButton.Ok,
        defaultButton=QtWidgets.QMessageBox.StandardButton.Ok,
        escapeButton=QtWidgets.QMessageBox.StandardButton.Ok,
        timeout=DEFAULT_TIMEOUT,
    ):
        msg = cls(
            QtWidgets.QMessageBox.Icon.Critical,
            title,
            text,
            buttons,
            parent,
            timeout=timeout,
            defaultButton=defaultButton,
            escapeButton=escapeButton,
        )

        return msg.exec()

    @classmethod
    def information(
        cls,
        parent,
        title,
        text,
        buttons=QtWidgets.QMessageBox.StandardButton.Ok,
        defaultButton=QtWidgets.QMessageBox.StandardButton.Ok,
        escapeButton=QtWidgets.QMessageBox.StandardButton.Ok,
        timeout=DEFAULT_TIMEOUT,
    ):
        msg = cls(
            QtWidgets.QMessageBox.Icon.Information,
            title,
            text,
            buttons,
            parent,
            timeout=timeout,
            defaultButton=defaultButton,
            escapeButton=escapeButton,
        )
        return msg.exec()

    @classmethod
    def question(
        cls,
        parent,
        title,
        text,
        buttons=QtWidgets.QMessageBox.StandardButton.Yes | QtWidgets.QMessageBox.StandardButton.No,
        defaultButton=QtWidgets.QMessageBox.StandardButton.No,
        escapeButton=QtWidgets.QMessageBox.StandardButton.No,
        timeout=DEFAULT_TIMEOUT,
    ):
        msg = cls(
            QtWidgets.QMessageBox.Icon.Question,
            title,
            text,
            buttons,
            parent,
            timeout=timeout,
            defaultButton=defaultButton,
            escapeButton=escapeButton,
        )
        return msg.exec()

    @classmethod
    def warning(
        cls,
        parent,
        title,
        text,
        buttons=QtWidgets.QMessageBox.StandardButton.Ok,
        defaultButton=QtWidgets.QMessageBox.StandardButton.Ok,
        escapeButton=QtWidgets.QMessageBox.StandardButton.Ok,
        timeout=DEFAULT_TIMEOUT,
    ):
        msg = cls(
            QtWidgets.QMessageBox.Icon.Warning,
            title,
            text,
            buttons,
            parent,
            timeout=timeout,
            defaultButton=defaultButton,
            escapeButton=escapeButton,
        )
        return msg.exec()
